import java.util.EmptyStackException;

public class ArraySimpleStack<E> implements SimpleStack<E> {
    /**
     * The default capacity of the stack.
     */
    public static final int DEFAULT_CAPACITY = 16;

    /**
     * The current capacity of the stack.
     */
    private int capacity;

    /**
     * The current size (i.e., number of elements contained in) the
     * stack.
     */
    private int size = 0;

    /**
     * The contents of the stack. The item currently at the top of the
     * stack is stored at index <code>size - 1</code>
     */
    private Object[] contents;

    /**
     * Create an <code>ArraySimpleList</code> with an initial capacity
     * of <code>DEFAULT_CAPACITY</code>.
     */
    public ArraySimpleStack() {
	this(DEFAULT_CAPACITY);
    }

    /**
     * Create an <code>ArraySimpleStack</code> with an initial capacity
     * of <code>initialCapacity</code>.
     *
     * @param initialCapacity the desired initial capacity of the stack
     */
    public ArraySimpleStack(int initialCapacity) {
	this.capacity = initialCapacity;
	contents = new Object[capacity];
    }

    @Override
    public int size() {
	return this.size;
    }

    @Override
    public boolean isEmpty() {
	return this.size == 0;
    }
    
    @Override
    public void push(E x) {
	if (size == capacity) {
	    increaseCapacity();
	}

	contents[size] = x;
	++size;
    }

    @SuppressWarnings("unchecked")
    @Override
    public E pop() {
	if (size == 0) {
	    throw new EmptyStackException();
	}

	--size;
	
	return (E) contents[size];
    }
    

    @SuppressWarnings("unchecked")
    @Override
    public E peek() {
	if (size == 0)
	    return null;

	return (E) contents[size - 1];
    }
    

    /**
     * Increase the capacity of the <code>ArraySimpleStack</code> by
     * copying the elements in <code>contents</code> into a larger
     * array and updating <code>capacity</code> to the size of the new
     * array.
     */
    private void increaseCapacity() {
	
	// create a new array with larger capacity
	Object[] bigContents = new Object[2 * capacity];

	// copy contents to bigContents
	for (int i = 0; i < capacity; ++i) {
	    bigContents[i] = contents[i];
	}

	// set contents to refer to the new array
	contents = bigContents;

	// update this.capacity accordingly
	capacity = 2 * capacity;
    }
    
}
